#
# Aeon Quantum Distortion Warhead, nuke launched
#
local AQuantumWarheadProjectile = import('/lua/aeonprojectiles.lua').AQuantumWarheadProjectile
local Entity = import('/lua/sim/Entity.lua').Entity

AntiNukeFlareLauncher = Class(Entity) {
    OnCreate = function(self, spec)
        Entity.OnCreate(self, spec)
        self.Owner = spec.Owner
        self.Radius = spec.Radius
        self.RedirectRateOfFire = spec.RedirectRateOfFire or 1
        self:SetCollisionShape('Sphere', 0, 0, 0, self.Radius)
        self:SetDrawScale(self.Radius)
        self.AttachBone = spec.AttachBone
        self:AttachTo(spec.Owner, spec.AttachBone)
    end,

        OnCollisionCheck = function(self, other)
            if (other:GetTrackingTarget() == self.Owner) and EntityCategoryContains(categories.ANTIMISSILE, other) and not EntityCategoryContains(categories.STRATEGIC, other)
                        and other != self.EnemyProj and IsEnemy( self:GetArmy(), other:GetArmy() ) and self.Owner.Charges >= 1 then
	local proj = self.Owner:CreateChildProjectile('/mods/conKORD imbalance pack/hook/projectiles/AIMAntiAntiNuke01/AIMAntiAntiNuke01_proj.bp')
                proj:SetVelocity(Random(-50,50),Random(-10,100),Random(-50,50))
                proj:SetVelocity(70)
                other:SetNewTarget(proj)
	local vx,vy,vz = other:GetVelocity()
                other:SetTurnRate(30)
                other:SetVelocity(vx*10+Random(-50,50),vy*10,vz*10+Random(-50,50))
                other:SetVelocity(80)
	self.Owner.Charges = self.Owner.Charges - 1
            end
            return false
        end,
}

AIFQuantumWarhead03 = Class(AQuantumWarheadProjectile) {

    Beams = {'/effects/emitters/aeon_nuke_exhaust_beam_02_emit.bp',},

    OnImpact = function(self, TargetType, TargetEntity)
        if not TargetEntity or not EntityCategoryContains(categories.PROJECTILE, TargetEntity) then
            # Play the explosion sound
            local myBlueprint = self:GetBlueprint()
            if myBlueprint.Audio.Explosion then
                self:PlaySound(myBlueprint.Audio.Explosion)
            end

            nukeProjectile = self:CreateProjectile('/projectiles/AIFQuantumWarhead02/AIFQuantumWarhead02_proj.bp', 0, 0, 0, nil, nil, nil):SetCollision(false)
            nukeProjectile:PassData(self.Data)
        end
        AQuantumWarheadProjectile.OnImpact(self, TargetType, TargetEntity)
    end,

    DoTakeDamage = function(self, instigator, amount, vector, damageType)
        if self.ProjectileDamaged then
            for k,v in self.ProjectileDamaged do
                v(self)
            end
        end
        AQuantumWarheadProjectile.DoTakeDamage(self, instigator, amount, vector, damageType)
    end,

        OnCollisionCheck = function(self, other)
	if self.Charges>0 then
		return false
	else
		return true
	end
        end,

    OnCreate = function(self)
        AQuantumWarheadProjectile.OnCreate(self)
        local launcher = self:GetLauncher()
        if launcher and not launcher:IsDead() and launcher.EventCallbacks.ProjectileDamaged then
            self.ProjectileDamaged = {}
            for k,v in launcher.EventCallbacks.ProjectileDamaged do
                table.insert( self.ProjectileDamaged, v )
            end
        end
        self:SetCollisionShape('Sphere', 0, 0, 0, 2.0)
        self.MovementTurnLevel = 1
        self:ForkThread( self.MovementThread )
    end,

    MovementThread = function(self)
        local army = self:GetArmy()
        local launcher = self:GetLauncher()
        self.idist = self:GetDistanceToTarget()  
        self.Charges = 1
        self:TrackTarget(false)
        WaitSeconds(2.5)		# Height
        self:SetCollision(true)
        self:SetDestroyOnWater(true)
        WaitSeconds(2.5)
        WaitSeconds(2.5)
        local antiMissile = AntiNukeFlareLauncher {
            Owner = self,
            Radius = 70,
            AttachBone = 'AIFQuantumWarhead01',
        }
                 self:TrackTarget(true) 
        self:SetDestroyOnWater(true)        
            self:SetMaxSpeed(160)
            self:SetAcceleration(10)
            WaitSeconds(5)
        self.WaitTime = 0.5
        while not self:BeenDestroyed() do
            self:SetTurnRateByDist()
            WaitSeconds(self.WaitTime)
        end
    end,

    SetTurnRateByDist = function(self)
        local dist = self:GetDistanceToTarget()
        local speed = self:GetCurrentSpeed()
        #print(speed*10,((114 * speed) / (self.idist^2)))
        if dist > 150  then
            self:SetMaxSpeed(160)
            self:SetTurnRate((1140 * speed) / (self.idist^2))
        elseif dist > 100 and dist < 150 then
            self:SetMaxSpeed(100)
            self.WaitTime = 0.3
            self:SetTurnRate((1140 * speed) / (self.idist^2))
        elseif dist > 32 and dist < 100 and self.idist > 90 then
            self:SetMaxSpeed(50)
            self:ChangeMaxZigZag(30)
            self:SetTurnRate(50)
            self.WaitTime = 0.1     
        elseif dist < 32 and self.idist > 90 then
            self:ChangeMaxZigZag(10)
            self:SetTurnRate(80)
        elseif dist < 100 and self.idist < 90 then
            self:ChangeMaxZigZag(10)
            self:SetTurnRate((1140 * speed) / (self.idist^2))
        end
    end,

    GetDistanceToTarget = function(self)
        local tpos = self:GetCurrentTargetPosition()
        local mpos = self:GetPosition()
        local dist = VDist2(mpos[1], mpos[3], tpos[1], tpos[3])
        return dist
    end,
}

TypeClass = AIFQuantumWarhead03

