#****************************************************************************
#**
#**  File     :  /data/projectiles/SIFInainoStrategicMissile02/SIFInainoStrategicMissile02_script.lua
#**  Author(s):  Gordon Duclos, Matt Vainio
#**
#**  Summary  :  Inaino Strategic Missile Projectile script, XSS0302
#**
#**  Copyright  2007 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************
local SIFInainoStrategicMissile = import('/lua/seraphimprojectiles.lua').SIFInainoStrategicMissile
local Entity = import('/lua/sim/Entity.lua').Entity

Teleporter = Class(Entity) {
    OnCreate = function(self, spec)
        Entity.OnCreate(self, spec)
        self.Owner = spec.Owner
        self.Radius = spec.Radius
        self.RedirectRateOfFire = spec.RedirectRateOfFire or 1
        self:SetCollisionShape('Sphere', 0, 0, 0, self.Radius)
        self:SetDrawScale(self.Radius)
        self.AttachBone = spec.AttachBone
        self:AttachTo(spec.Owner, spec.AttachBone)
    end,

        OnCollisionCheck = function(self, other)
            if (other:GetTrackingTarget() == self.Owner) and EntityCategoryContains(categories.ANTIMISSILE, other) and not EntityCategoryContains(categories.STRATEGIC, other)
                        and other != self.EnemyProj and IsEnemy( self:GetArmy(), other:GetArmy() ) and self.Owner.Charges >= 1 then
	local pos = self.Owner:GetPosition()
                self.EnemyProj = other
                other:SetTurnRate(30)
	local army = self:GetArmy()
 	CreateLightParticle( self, -1, army, 18, 1, 'flare_lens_add_02', 'ramp_blue_13' )
	CreateEmitterOnEntity(self, army, '/effects/Emitters/seraphim_khamaseen_bomb_hit_11_emit.bp')
	CreateEmitterOnEntity(self, army, '/effects/Emitters/seraphim_khamaseen_bomb_hit_12_emit.bp')
	CreateEmitterOnEntity(self, army, '/effects/Emitters/seraphim_khamaseen_bomb_hit_10_emit.bp')
	Warp(self.Owner, {pos[1]+Random(-20,20),pos[2],pos[3]+Random(-20,20)})
	self.Owner.Charges = self.Owner.Charges -1
	CreateLightParticle( self, -1, army, 18, 1, 'flare_lens_add_02', 'ramp_blue_13' )
            end
            return false
        end,
}

SIFInainoStrategicMissile02 = Class(SIFInainoStrategicMissile) {

    FxSplashScale = 0.5,
    FxTrails = {},

    LaunchSound = 'Nuke_Launch',
    ExplodeSound = 'Nuke_Impact',
    AmbientSound = 'Nuke_Flight',

	InitialEffects = {
		'/effects/emitters/seraphim_inaino_fxtrails_01_emit.bp',
		'/effects/emitters/seraphim_inaino_fxtrails_02_emit.bp',
	},    
    ThrustEffects = {
		'/effects/emitters/seraphim_inaino_fxtrails_01_emit.bp',
		'/effects/emitters/seraphim_inaino_fxtrails_02_emit.bp',
	},
    LaunchEffects = {
		'/effects/emitters/seraphim_inaino_fxtrails_01_emit.bp',
		'/effects/emitters/seraphim_inaino_fxtrails_02_emit.bp',
	},
    
    OnCreate = function(self)
        SIFInainoStrategicMissile.OnCreate(self)
        local launcher = self:GetLauncher()
        if launcher and not launcher:IsDead() and launcher.EventCallbacks.ProjectileDamaged then
            self.ProjectileDamaged = {}
            for k,v in launcher.EventCallbacks.ProjectileDamaged do
                table.insert( self.ProjectileDamaged, v )
            end
        end 
        self:SetCollisionShape('Sphere', 0, 0, 0, 2.0)
        self.MovementTurnLevel = 1
        self:ForkThread( self.MovementThread )
    end,    

    OnImpact = function(self, TargetType, TargetEntity)
        if not TargetEntity or not EntityCategoryContains(categories.PROJECTILE, TargetEntity) then
            # Play the explosion sound
            local myBlueprint = self:GetBlueprint()
            if myBlueprint.Audio.Explosion then
                self:PlaySound(myBlueprint.Audio.Explosion)
            end
    
            nukeProjectile = self:CreateProjectile('/effects/entities/InainoEffectController01/InainoEffectController01_proj.bp', 0, 0, 0, nil, nil, nil):SetCollision(false)
            nukeProjectile:PassData(self.Data)
        end
        SIFInainoStrategicMissile.OnImpact(self, TargetType, TargetEntity)
    end,

    DoTakeDamage = function(self, instigator, amount, vector, damageType)
        if self.ProjectileDamaged then
            for k,v in self.ProjectileDamaged do
                v(self)
            end
        end
        SIFInainoStrategicMissile.DoTakeDamage(self, instigator, amount, vector, damageType)
    end,

        OnCollisionCheck = function(self, other)
	if self.Charges>0 then
		return false
	else
		return true
	end
        end,

    CreateEffects = function( self, EffectTable, army, scale)
        for k, v in EffectTable do
            self.Trash:Add(CreateAttachedEmitter(self, -1, army, v):ScaleEmitter(scale))
        end
    end,

    MovementThread = function(self)
        local army = self:GetArmy()
        local launcher = self:GetLauncher()
        self.idist = self:GetDistanceToTarget()  
        self.Charges = 1
        self.CreateEffects( self, self.InitialEffects, army, 1 )
        self:TrackTarget(false)
        WaitSeconds(2)
        self:SetCollision(true)
        self.CreateEffects( self, self.LaunchEffects, army, 1 )
        WaitSeconds(2)
        self.CreateEffects( self, self.ThrustEffects, army, 3 )
        WaitSeconds(2)
        local antiMissile = Teleporter {
            Owner = self,
            Radius = 10,
            AttachBone = 'SIFExperimental_Strategic_Launcher',
        }
               self:TrackTarget(true) 
        self:SetDestroyOnWater(true)        
            self:SetMaxSpeed(160)
            self:SetAcceleration(10)
            WaitSeconds(5)
        self.WaitTime = 0.5
        while not self:BeenDestroyed() do
            self:SetTurnRateByDist()
            WaitSeconds(self.WaitTime)
        end
    end,

    SetTurnRateByDist = function(self)
        local dist = self:GetDistanceToTarget()
        local speed = self:GetCurrentSpeed()
        #print(speed*10,((114 * speed) / (self.idist^2)))
        if dist > 150  then
            self:SetMaxSpeed(160)
            self:SetTurnRate((1140 * speed) / (self.idist^2))
        elseif dist > 100 and dist < 150 then
            self:SetMaxSpeed(100)
            self.WaitTime = 0.3
            self:SetTurnRate((1140 * speed) / (self.idist^2))
        elseif dist > 32 and dist < 100 and self.idist > 90 then
            self:SetMaxSpeed(50)
            self:ChangeMaxZigZag(30)
            self:SetTurnRate(50)
            self.WaitTime = 0.1     
        elseif dist < 32 and self.idist > 90 then
            self:ChangeMaxZigZag(10)
            self:SetTurnRate(80)
        elseif dist < 100 and self.idist < 90 then
            self:ChangeMaxZigZag(10)
            self:SetTurnRate((1140 * speed) / (self.idist^2))
        end
    end,


    GetDistanceToTarget = function(self)
        local tpos = self:GetCurrentTargetPosition()
        local mpos = self:GetPosition()
        local dist = VDist2(mpos[1], mpos[3], tpos[1], tpos[3])
        return dist
    end,

}

TypeClass = SIFInainoStrategicMissile02
